@@ -226,6 +226,10 @@ pulseaudio_button_button_press (GtkWidget      *widget,
              G_CALLBACK (pulseaudio_button_menu_deactivate), button);
         }
 
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 17, 2)
+      xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (button->plugin), GTK_MENU (button->menu),
+                                    widget, (GdkEvent *) event);
+#else
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_menu_popup (GTK_MENU (button->menu),
                       NULL, NULL,
@@ -233,6 +237,7 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                       1,
                       event->time);
 G_GNUC_END_IGNORE_DEPRECATIONS
+#endif
 
       return TRUE;
     }
@@ -329,6 +334,7 @@ pulseaudio_button_update (PulseaudioButton *button,
   gboolean     connected;
   gboolean     sink_connected;
   gboolean     muted;
+  gboolean     recording;
   gchar       *tip_text;
   const gchar *icon_name;
 
@@ -339,6 +345,7 @@ pulseaudio_button_update (PulseaudioButton *button,
   muted = pulseaudio_volume_get_muted (button->volume);
   connected = pulseaudio_volume_get_connected (button->volume);
   sink_connected = pulseaudio_volume_get_sink_connected (button->volume);
+  recording = pulseaudio_volume_get_recording (button->volume);
 
   if (!connected)
     icon_name = icons[V_MUTED];
@@ -369,6 +376,9 @@ pulseaudio_button_update (PulseaudioButton *button,
       gtk_image_set_pixel_size (GTK_IMAGE (button->image), button->icon_size);
     }
 
+  if (gtk_widget_get_visible (button->recording_indicator) != recording)
+    gtk_widget_set_visible (button->recording_indicator, recording);
+
   if (!sink_connected)
     g_timeout_add (250, pulseaudio_button_sink_connection_timeout, button);
 }
@@ -405,6 +415,8 @@ pulseaudio_button_recording_changed (PulseaudioButton *button,
                                      gboolean          recording,
                                      PulseaudioVolume *volume)
 {
+  g_return_if_fail (IS_PULSEAUDIO_BUTTON (button));
+
   /* Show or hide the recording indicator */
   gtk_widget_set_visible (button->recording_indicator, recording);
 }
