@@ -1,4 +1,4 @@
-/*  Copyright (c) 2017 Sean Davis <bluesabre@xfce.org>
+/*  Copyright (c) 2017-2020 Sean Davis <bluesabre@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -67,18 +67,21 @@ G_DEFINE_TYPE (PulseaudioMpris, pulseaudio_mpris, G_TYPE_OBJECT)
 
 
 
+static PulseaudioMpris *mpris_instance;
+
+
+
 static gchar *
 find_desktop_entry (const gchar *player_name)
 {
   GKeyFile  *key_file;
   gchar     *file = NULL;
   gchar     *filename = NULL;
-  gchar     *full_path = NULL;
 
   file = g_strconcat ("applications/", player_name, ".desktop", NULL);
 
   key_file = g_key_file_new();
-  if (g_key_file_load_from_data_dirs (key_file, file, &full_path, G_KEY_FILE_NONE, NULL))
+  if (g_key_file_load_from_data_dirs (key_file, file, NULL, G_KEY_FILE_NONE, NULL))
     {
       filename = g_strconcat (player_name, ".desktop", NULL);
     }
@@ -107,9 +110,6 @@ find_desktop_entry (const gchar *player_name)
   if (file)
     g_free (file);
 
-  if (full_path)
-    g_free (full_path);
-
   return filename;
 }
 
@@ -134,6 +134,40 @@ pulseaudio_mpris_class_init (PulseaudioMprisClass *klass)
 }
 
 
+static gboolean
+player_is_usable (PulseaudioMpris *mpris,
+                  const gchar     *player) {
+  GVariantIter iter;
+  GVariant *result, *child = NULL;
+  gboolean usable = FALSE;
+
+  result = g_dbus_connection_call_sync (mpris->dbus_connection,
+                                        player,
+                                        "/org/mpris/MediaPlayer2",
+                                        "org.freedesktop.DBus.Properties",
+                                        "GetAll",
+                                        g_variant_new ("(s)", "org.mpris.MediaPlayer2"),
+                                        G_VARIANT_TYPE ("(a{sv})"),
+                                        G_DBUS_CALL_FLAGS_NONE,
+                                        -1,
+                                        NULL,
+                                        NULL);
+
+  if (result)
+    {
+      g_variant_iter_init (&iter, result);
+      child = g_variant_iter_next_value (&iter);
+      if (child)
+        {
+          usable = TRUE;
+          g_variant_unref (child);
+        }
+      g_variant_unref (result);
+    }
+
+  return usable;
+}
+
 
 gchar **
 pulseaudio_mpris_get_available_players (PulseaudioMpris *mpris)
@@ -169,9 +203,11 @@ pulseaudio_mpris_get_available_players (PulseaudioMpris *mpris)
     {
       if (g_str_has_prefix(str, "org.mpris.MediaPlayer2."))
         {
-          res = (gchar**)g_realloc(res, (items + 1) * sizeof(gchar*));
-          res[items] = g_strdup(str + 23);
-          items++;
+          if (player_is_usable (mpris, str)) {
+            res = (gchar**)g_realloc(res, (items + 1) * sizeof(gchar*));
+            res[items] = g_strdup(str + 23);
+            items++;
+          }
         }
     }
 
@@ -313,18 +349,18 @@ pulseaudio_mpris_get_player_snapshot (PulseaudioMpris  *mpris,
 
 
 
-gboolean
-pulseaudio_mpris_get_player_summary (const gchar  *player,
-                                     gchar       **name,
-                                     gchar       **icon_name,
-                                     gchar       **full_path)
+static gboolean
+pulseaudio_mpris_get_player_summary_from_desktop (const gchar  *player_id,
+                                                  gchar       **name,
+                                                  gchar       **icon_name,
+                                                  gchar       **full_path)
 {
   GKeyFile  *key_file;
   gchar     *file;
   gchar     *filename;
   gchar     *path;
 
-  filename = find_desktop_entry (player);
+  filename = find_desktop_entry (player_id);
   if (filename == NULL)
     {
       return FALSE;
@@ -350,6 +386,33 @@ pulseaudio_mpris_get_player_summary (const gchar  *player,
 
 
 
+gboolean
+pulseaudio_mpris_get_player_summary (const gchar  *player_id,
+                                     gchar       **name,
+                                     gchar       **icon_name,
+                                     gchar       **full_path)
+{
+  PulseaudioMprisPlayer *player;
+  player = PULSEAUDIO_MPRIS_PLAYER (g_hash_table_lookup (mpris_instance->players, player_id));
+
+  if (player == NULL)
+    return pulseaudio_mpris_get_player_summary_from_desktop (player_id, name, icon_name, full_path);
+
+  *name = g_strdup (pulseaudio_mpris_player_get_player_title (player));
+  *icon_name = g_strdup (pulseaudio_mpris_player_get_icon_name (player));
+  *full_path = g_strdup (pulseaudio_mpris_player_get_full_path (player));
+
+  if (*full_path == NULL)
+    return FALSE;
+
+  if (!pulseaudio_mpris_player_is_connected (player) && !pulseaudio_mpris_player_can_launch (player))
+    return FALSE;
+
+  return TRUE;
+}
+
+
+
 gboolean
 pulseaudio_mpris_notify_player (PulseaudioMpris  *mpris,
                                 const gchar      *name,
@@ -480,5 +543,7 @@ pulseaudio_mpris_new (PulseaudioConfig *config)
   mpris->players = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
   mpris->player_timer_id = g_timeout_add_seconds (1, pulseaudio_mpris_tick_cb, mpris);
 
+  mpris_instance = mpris;
+
   return mpris;
 }
