@@ -60,7 +60,9 @@
 #define PULSEAUDIO_PLUGIN_NEXT_KEY          "XF86AudioNext"
 #endif
 
-
+#ifdef HAVE_LIBCANBERRA
+#include <canberra.h>
+#endif
 
 /* prototypes */
 static void             pulseaudio_plugin_init_debug                       (void);
@@ -123,6 +125,10 @@ struct _PulseaudioPlugin
 
   /* mpris */
   PulseaudioMpris     *mpris;
+
+#ifdef HAVE_LIBCANBERRA
+  ca_context          *canberra;
+#endif
 };
 
 
@@ -163,6 +169,10 @@ pulseaudio_plugin_init (PulseaudioPlugin *pulseaudio_plugin)
 #endif
 
   pulseaudio_plugin->mpris             = NULL;
+
+#ifdef HAVE_LIBCANBERRA
+  pulseaudio_plugin->canberra          = NULL;
+#endif
 }
 
 
@@ -177,6 +187,10 @@ pulseaudio_plugin_free_data (XfcePanelPlugin *plugin)
   pulseaudio_plugin_unbind_keys (pulseaudio_plugin);
   pulseaudio_plugin_unbind_multimedia_keys (pulseaudio_plugin);
 #endif
+#ifdef HAVE_LIBCANBERRA
+  if (pulseaudio_plugin->canberra)
+    ca_context_destroy (pulseaudio_plugin->canberra);
+#endif
 }
 
 
@@ -227,6 +241,7 @@ pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
       "Steve Dodier-Lazaro <sidi@xfce.org>",
       "Viktor Odintsev <zakhams@gmail.com>",
       "Matthieu Mota <matthieumota@gmail.com>",
+      "Sean Davis <bluesabre@xfce.org>",
       NULL
     };
 
@@ -239,8 +254,8 @@ pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
                          "version",      PACKAGE_VERSION,
                          "program-name", PACKAGE_NAME,
                          "comments",     _("Adjust the audio volume of the PulseAudio sound system"),
-                         "website",      "http://goodies.xfce.org/projects/panel-plugins/xfce4-pulseaudio-plugin",
-                         "copyright",    _("Copyright \xc2\xa9 2014-2017 Andrzej Radecki et al.\n"),
+                         "website",      "https://docs.xfce.org/panel-plugins/xfce4-pulseaudio-plugin",
+                         "copyright",    _("Copyright \xc2\xa9 2014-2022 Andrzej Radecki et al.\n"),
                          "authors",      auth,
                          NULL);
 
@@ -338,8 +353,6 @@ pulseaudio_plugin_unbind_keys (PulseaudioPlugin      *pulseaudio_plugin)
   keybinder_unbind (PULSEAUDIO_PLUGIN_MIC_MUTE_KEY, pulseaudio_plugin_mic_mute_pressed);
 }
 
-
-
 static void
 pulseaudio_plugin_volume_key_pressed (const char            *keystring,
                                       void                  *user_data)
@@ -350,10 +363,19 @@ pulseaudio_plugin_volume_key_pressed (const char            *keystring,
 
   pulseaudio_debug ("%s pressed", keystring);
 
-  if (strcmp (keystring, PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY) == 0)
+  if (strcmp (keystring, PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY) == 0) {
     pulseaudio_volume_set_volume (pulseaudio_plugin->volume, MIN (volume + volume_step, MAX (volume, 1.0)));
-  else if (strcmp (keystring, PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY) == 0)
+#ifdef HAVE_LIBNOTIFY
+    /* Also send notification when volume is already at 100% */
+    pulseaudio_notify_volume_changed (pulseaudio_plugin->notify, 1, pulseaudio_plugin->volume);
+#endif
+  } else if (strcmp (keystring, PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY) == 0) {
     pulseaudio_volume_set_volume (pulseaudio_plugin->volume, volume - volume_step);
+#ifdef HAVE_LIBNOTIFY
+    /* Also send notification when volume is already at 0% */
+    pulseaudio_notify_volume_changed (pulseaudio_plugin->notify, 1, pulseaudio_plugin->volume);
+#endif
+  }
 }
 
 
@@ -485,6 +507,26 @@ pulseaudio_plugin_next_key_pressed (const char            *keystring,
 
 
 
+#ifdef HAVE_LIBCANBERRA
+void
+pulseaudio_plugin_play_sound (PulseaudioPlugin      *pulseaudio_plugin,
+                              const char            *event_id,
+                              const char            *event_desc)
+{
+  pulseaudio_debug("play %s", event_id);
+
+  if (pulseaudio_config_get_play_sound (pulseaudio_plugin->config))
+    {
+      ca_context_play (pulseaudio_plugin->canberra, 0,
+                       CA_PROP_EVENT_ID, event_id,
+                       CA_PROP_EVENT_DESCRIPTION, event_desc,
+                       NULL);
+    }
+}
+#endif
+
+
+
 static void
 pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
 {
@@ -524,7 +566,8 @@ pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
 #endif
 
   /* volume controller */
-  pulseaudio_plugin->volume = pulseaudio_volume_new (pulseaudio_plugin->config);
+  pulseaudio_plugin->volume = pulseaudio_volume_new (pulseaudio_plugin,
+                                                     pulseaudio_plugin->config);
 
   /* initialize mpris support */
 #ifdef HAVE_MPRIS2
@@ -544,6 +587,11 @@ pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
                                                      pulseaudio_plugin->button);
 #endif
 
+  /* initialize canberra support */
+#ifdef HAVE_LIBCANBERRA
+  ca_context_create (&pulseaudio_plugin->canberra);
+#endif
+
   gtk_container_add (GTK_CONTAINER (plugin), GTK_WIDGET (pulseaudio_plugin->button));
   gtk_widget_show (GTK_WIDGET (pulseaudio_plugin->button));
 }
