@@ -38,25 +38,29 @@
 #include "pulseaudio-volume.h"
 
 
-static void                 pulseaudio_volume_finalize           (GObject              *object);
-static void                 pulseaudio_volume_connect            (PulseaudioVolume     *volume);
-static gdouble              pulseaudio_volume_v2d                (PulseaudioVolume     *volume,
-                                                                  pa_volume_t           vol);
-static pa_volume_t          pulseaudio_volume_d2v                (PulseaudioVolume     *volume,
-                                                                  gdouble               vol);
-static gboolean             pulseaudio_volume_reconnect_timeout  (gpointer              userdata);
-static void                 pulseaudio_volume_get_sink_list_cb   (pa_context           *context,
-                                                                  const pa_sink_info   *i,
-                                                                  int                   eol,
-                                                                  void                 *userdata);
-static void                 pulseaudio_volume_get_source_list_cb (pa_context           *context,
-                                                                  const pa_source_info *i,
-                                                                  int                   eol,
-                                                                  void                 *userdata);
-static void                 pulseaudio_volume_move_sink_input    (pa_context           *context,
-                                                                  const                 pa_sink_input_info *i,
-                                                                  int                   eol,
-                                                                  void                 *userdata);
+static void                 pulseaudio_volume_finalize                  (GObject              *object);
+static void                 pulseaudio_volume_connect                   (PulseaudioVolume     *volume);
+static gdouble              pulseaudio_volume_v2d                       (PulseaudioVolume     *volume,
+                                                                         pa_volume_t           vol);
+static pa_volume_t          pulseaudio_volume_d2v                       (PulseaudioVolume     *volume,
+                                                                         gdouble               vol);
+static gboolean             pulseaudio_volume_reconnect_timeout         (gpointer              userdata);
+static void                 pulseaudio_volume_get_sink_list_cb          (pa_context           *context,
+                                                                         const pa_sink_info   *i,
+                                                                         int                   eol,
+                                                                         void                 *userdata);
+static void                 pulseaudio_volume_get_source_list_cb        (pa_context           *context,
+                                                                         const pa_source_info *i,
+                                                                         int                   eol,
+                                                                         void                 *userdata);
+static void                 pulseaudio_volume_get_source_output_info_cb (pa_context           *context,
+                                                                         const pa_source_output_info *i,
+                                                                         int                   eol,
+                                                                         void                 *userdata);
+static void                 pulseaudio_volume_move_sink_input           (pa_context           *context,
+                                                                         const                 pa_sink_input_info *i,
+                                                                         int                   eol,
+                                                                         void                 *userdata);
 
 
 
@@ -64,6 +68,7 @@ struct _PulseaudioVolume
 {
   GObject               __parent__;
 
+  PulseaudioPlugin     *plugin;
   PulseaudioConfig     *config;
 
   pa_glib_mainloop     *pa_mainloop;
@@ -178,6 +183,7 @@ pulseaudio_volume_finalize (GObject *object)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (object);
 
+  volume->plugin = NULL;
   volume->config = NULL;
 
   if (volume->default_sink_name)
@@ -269,7 +275,7 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
       volume->muted_mic = muted_mic;
 
       if (volume->source_connected)
-        g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+        g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, TRUE);
     }
 
   if (ABS (volume->volume_mic - vol_mic) > 2e-3)
@@ -278,7 +284,7 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
       volume->volume_mic = vol_mic;
 
       if (volume->source_connected)
-        g_signal_emit(G_OBJECT(volume), pulseaudio_volume_signals[VOLUME_MIC_CHANGED], 0, FALSE);
+        g_signal_emit(G_OBJECT(volume), pulseaudio_volume_signals[VOLUME_MIC_CHANGED], 0, TRUE);
     }
 
   pulseaudio_debug ("volume mic: %f, muted mic: %d", vol_mic, muted_mic);
@@ -311,23 +317,15 @@ static void
 pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
                                      pa_context       *context)
 {
-  gboolean recording;
   g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
 
   pa_context_get_server_info (context, pulseaudio_volume_server_info_cb, volume);
 
-  recording = volume->recording;
-  volume->recording = FALSE;
-
   g_hash_table_remove_all (volume->sinks);
   g_hash_table_remove_all (volume->sources);
 
   pa_context_get_sink_info_list (volume->pa_context, pulseaudio_volume_get_sink_list_cb, volume);
   pa_context_get_source_info_list (volume->pa_context, pulseaudio_volume_get_source_list_cb, volume);
-
-  /* Send a signal because the recording state has changed */
-  if (volume->recording != recording)
-    g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, volume->recording);
 }
 
 
@@ -339,6 +337,7 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
                                 void                         *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  gboolean recording;
 
   switch (t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK)
     {
@@ -358,6 +357,13 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
       break;
 
     case PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT :
+      recording = volume->recording;
+      volume->recording = FALSE;
+      pa_context_get_source_output_info_list (context, pulseaudio_volume_get_source_output_info_cb, volume);
+
+      if (volume->recording != recording)
+        g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, volume->recording);
+
       pulseaudio_volume_sink_source_check (volume, context);
       pulseaudio_debug ("received source output event");
       break;
@@ -394,6 +400,36 @@ pulseaudio_volume_get_sink_list_cb (pa_context         *context,
 
 
 
+static void
+pulseaudio_volume_get_source_output_info_cb (pa_context                  *context,
+                                             const pa_source_output_info *i,
+                                             int                          eol,
+                                             void                        *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+
+  if (eol > 0)
+    return;
+
+  if (i)
+    {
+      if (g_strcmp0("org.PulseAudio.pavucontrol",
+                    pa_proplist_gets (i->proplist, PA_PROP_APPLICATION_ID)) == 0)
+        {
+          pulseaudio_debug ("Don't show recording indicator for pavucontrol");
+          return;
+        }
+
+      volume->recording = TRUE;
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, TRUE);
+    }
+  else
+    {
+      volume->recording = FALSE;
+    }
+}
+
+
 static void
 pulseaudio_volume_get_source_list_cb (pa_context           *context,
                                       const pa_source_info *i,
@@ -412,13 +448,6 @@ pulseaudio_volume_get_source_list_cb (pa_context           *context,
   if (i->monitor_of_sink != PA_INVALID_INDEX)
     return;
 
-  if (i->state == PA_SOURCE_RUNNING)
-  {
-    pulseaudio_debug ("Source %s is recording", i->name);
-    volume->recording = TRUE;
-    g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, TRUE);
-  }
-
   g_hash_table_insert (volume->sources, g_strdup (i->name), g_strdup (i->description));
 }
 
@@ -473,7 +502,7 @@ pulseaudio_volume_context_state_cb (pa_context *context,
 
     case PA_CONTEXT_FAILED       :
     case PA_CONTEXT_TERMINATED   :
-      g_warning ("Disconected from the PulseAudio server. Attempting to reconnect in 5 seconds.");
+      g_warning ("Disconnected from the PulseAudio server. Attempting to reconnect in 5 seconds...");
       volume->pa_context = NULL;
       volume->connected = FALSE;
       volume->volume = 0.0;
@@ -511,7 +540,7 @@ pulseaudio_volume_context_state_cb (pa_context *context,
       break;
 
     default                      :
-      g_warning ("Unknown pulseaudio context state");
+      g_warning ("Unknown PulseAudio context state");
       break;
     }
 }
@@ -635,6 +664,16 @@ pulseaudio_volume_get_muted (PulseaudioVolume *volume)
 
 
 
+gboolean
+pulseaudio_volume_get_recording (PulseaudioVolume *volume)
+{
+  g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), FALSE);
+
+  return volume->recording;
+}
+
+
+
 /* final callback for volume/mute changes */
 /* pa_context_success_cb_t */
 static void
@@ -645,7 +684,12 @@ pulseaudio_volume_sink_volume_changed (pa_context *context,
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
 
   if (success)
-    g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, TRUE);
+    {
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, TRUE);
+#ifdef HAVE_LIBCANBERRA
+      pulseaudio_plugin_play_sound (volume->plugin, "audio-volume-change", "volume changed");
+#endif
+    }
 }
 
 
@@ -1053,13 +1097,15 @@ pulseaudio_volume_set_default_input (PulseaudioVolume *volume,
 
 
 PulseaudioVolume *
-pulseaudio_volume_new (PulseaudioConfig *config)
+pulseaudio_volume_new (PulseaudioPlugin *plugin,
+                       PulseaudioConfig *config)
 {
   PulseaudioVolume *volume;
 
   g_return_val_if_fail (IS_PULSEAUDIO_CONFIG (config), NULL);
 
   volume = g_object_new (TYPE_PULSEAUDIO_VOLUME, NULL);
+  volume->plugin = plugin;
   volume->config = config;
 
   return volume;
